<?php
/**
 * Mandae Webservice.
 *
 * @package WooCommerce_Mandae/Classes/Webservice
 * @since   3.0.0
 * @version 1.0.1
 */

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Mandae Webservice integration class.
 */
class WC_Mandae_Webservice
{
    const SANDBOX_URL = 'https://sandbox.api.mandae.com.br/';

    const LIVE_URL = 'https://api.mandae.com.br/';

    /**
     * Shipping method ID.
     *
     * @var string
     */
    protected $id = '';

    /**
     * Shipping zone instance ID.
     *
     * @var int
     */
    protected $instance_id = 0;

    /**
     * Authorization.
     *
     * @var string
     */
    protected $auth = '';

    /**
     * Authorization.
     *
     * @var string
     */
    protected $environment = '';

    /**
     * WooCommerce package containing the products.
     *
     * @var array
     */
    protected $package = null;

    /**
     * Destination postcode.
     *
     * @var string
     */
    protected $destination_postcode = '';

    /**
     * Debug mode.
     *
     * @var string
     */
    protected $debug = 'no';

    /**
     * Logger.
     *
     * @var WC_Logger
     */
    public $log = null;

    /**
     * Whether to use declared value or not
     *
     * @var bool
     */
    private $use_declared_value = false;

    /**
     * Initialize webservice.
     *
     * @param string $id Method ID.
     * @param int $instance_id Instance ID.
     */
    public function __construct($id = 'mandae', $instance_id = 0)
    {
        $this->id          = $id;
        $this->instance_id = $instance_id;
        $this->log         = new WC_Logger();
        $this->use_declared_value = get_option('woocommerce_mandae-integration_settings', $default = false )['declared_value'];
    }

    /**
     * Set auth.
     *
     * @param string $auth Authorization.
     */
    public function set_auth($auth = '')
    {
        $this->auth = $auth;
    }

    /**
     * Set environment.
     *
     * @param string $environment Environment.
     */
    public function set_environment($environment = '')
    {
        $this->environment = $environment;
    }

    /**
     * Set shipping package.
     *
     * @param array $package Shipping package.
     */
    public function set_package($package = array())
    {
        $this->package = $package;
    }

    /**
     * Set destination postcode.
     *
     * @param string $postcode Destination postcode.
     */
    public function set_destination_postcode($postcode = '')
    {
        $this->destination_postcode = $postcode;
    }

    /**
     * Set the debug mode.
     *
     * @param string $debug Yes or no.
     */
    public function set_debug($debug = 'no')
    {
        $this->debug = $debug;
    }

    /**
     * Get authorization.
     *
     * @return string
     */
    public function get_auth()
    {
        return apply_filters('woocommerce_mandae_auth', $this->auth, $this->id, $this->instance_id, $this->package);
    }

    /**
     * Get environment.
     *
     * @return string
     */
    public function get_environment()
    {
        return apply_filters('woocommerce_mandae_environment', $this->environment, $this->id, $this->instance_id, $this->package);
    }

    /**
     * @param $environment
     * @return string
     */
    private function get_url($environment)
    {
        if ($environment == 'sandbox') {
            return self::SANDBOX_URL;
        }

        return self::LIVE_URL;
    }

    /**
     * Get shipping prices.
     *
     * @return SimpleXMLElement|array
     */
    public function get_shipping()
    {
        $url = $this->get_url($this->environment);

        $url .= 'v3/postalcodes/' . str_replace("-", "", $this->destination_postcode) . '/rates';

        $items = array_map(function ($item) {
            $product = $item['data'];

            $_height = max(1, wc_get_dimension((float)$product->get_length(), 'cm'));
            $_width  = max(1, wc_get_dimension((float)$product->get_width(), 'cm'));
            $_length = max(1, wc_get_dimension((float)$product->get_height(), 'cm'));
            $_weight = max(0.000001, wc_get_weight((float)$product->get_weight(), 'kg'));

            $itemData = [
                'quantity' => $item['quantity'],
                'weight' => $_weight,
                'height' => $_height,
                'width' => $_width,
                'length' => $_length
            ];

            if ($this->use_declared_value) {
                $itemData['declaredValue'] = $product->get_price();
            }

            return $itemData;
        }, array_values($this->package['contents']));

        $body = json_encode([
            'items' => $items
        ]);

        $this->log->add($this->id, 'BODY: ' . $body);
        $shipping = null;

        try {
            $response = wp_remote_post(
                $url,
                array(
                    'headers' => array(
                        'Content-Type'  => 'application/json',
                        'Authorization' => $this->auth
                    ),
                    'body' => $body
                )
            );

            $this->log->add($this->id, json_encode($response));

            if (!is_wp_error($response)) {
                $shipping = $response['body'];
            }
        } catch (Exception $e) {
            $this->log->add($this->id, $e->getMessage());
            return null;
        }

        return $shipping;
    }
}
